# -*- coding:utf-8 -*-

import time

import serial

from elio.comm.eliochannel import eliochannel
from elio.comm.elioprotocol import ElioProtocol
from elio.comm.packet_t import packet_t


if __name__ == "__main__":

    PORT = '/dev/tty.usbmodem4ABD0733AB9D2'
    ser = serial.serial_for_url(PORT, baudrate=115200, timeout=1)

    with eliochannel(ser, ElioProtocol, packet_t) as p:

        p.decideToUseSensor(1, 0, 0)
        while p.isDone():
            p.sendDC(-90, 0)
            time.sleep(1)
            p.sendDC(0, 0)
            time.sleep(1)

            p.sendServo(50, 0)
            time.sleep(1)
            p.sendServo(0, 0)
            time.sleep(1)

            p.sendIO("IO4", 100)
            time.sleep(1)
            # p.sendIO("IO4", 0)
            time.sleep(1)



# -*- coding:utf-8 -*-
import binascii

from elio.comm.protocol import Protocol
import struct

UDP = 0x30
CMD_EXECUTE = 0x01


class ElioProtocol(Protocol):
    # __dc1 = None;
    # __dc2 = None;
    # __sv1 = None;
    # __sv2 = None;
    # __v3 = None;
    # __v5 = None;
    # __io1 = None;
    # __io2 = None;
    # __io3 = None;
    # __io4 = None;
    #
    # __ultra = None;
    # __line1 = None;
    # __line2 = None;

    def __init__(self):
        self.initialize()
        self.ultra = 0
        self.line1 = 0
        self.line2 = 0

    def initialize(self):
        self.dc1 = 0
        self.dc2 = 0
        self.sv1 = 0
        self.sv2 = 0
        self.v3 = 0
        self.v5 = 0
        self.io1 = 0
        self.io2 = 0
        self.io3 = 0
        self.io4 = 0

    def decideToUseSensor(self, ultra, line1, line2):
        self.ultra = ultra
        self.line1 = line1
        self.line2 = line2

        self.sendDeviceData()

    def connection_made(self, transport):
        self.transport = transport
        self.running = True

    def connection_lost(self, exc):
        self.transport = None

    def data_received(self, data, len):
        cmd = data[0]
        udp = data[1]

        self.DC1 = data[2]
        self.DC2 = data[3]

        self.SV1 = data[4]
        self.SV2 = data[5]

        self.V3 = data[6]
        self.V5 = data[7]

        self.IO1 = data[8]
        self.IO2 = data[9]
        self.IO3 = data[10]
        self.IO4 = data[11]

        self.SONIC = (data[12] | data[13] << 8)
        self.LINE1 = (data[14] | data[15] << 8) == 0 if 1 else 0
        self.LINE2 = (data[16] | data[17] << 8) == 0 if 1 else 0

        print("dc1=", self.dc1, ", dc2=", self.dc2, ", sv1=", self.sv1, ", sv2=", self.sv2,
              ", io1=", self.io1, ", io2=", self.io2, ", io3=", self.io3, ", io4=", self.io4,
              ", SONIC=", self.SONIC, ", LINE1=", self.LINE1, ", LINE2=", self.LINE2)

    def write(self, data, len):
        # print(binascii.hexlify(data))
        self.transport.packet.send_packet(data, len)

    def write_packet(self, data):
        # print(data)
        self.transport.write(data)

    def isDone(self):
        return self.running

    def initializeData(self):
        pass
        # init = bytearray([0x20, 0x50, 0x00, 0x00, 0x00])
        # p.write(init)

    def sendIO(self, which_io, value):
        print('sendIO')
        if (which_io == "3V"):
            self.v3 = value
        elif (which_io == "5V"):
            self.v5 = value
        elif (which_io == "IO1"):
            self.io1 = value
        elif (which_io == "IO2"):
            self.io2 = value
        elif (which_io == "IO3"):
            self.io3 = value
        elif (which_io == "IO4"):
            self.io4 = value

        self.sendDeviceData()

    def sendDC(self, dc1, dc2):
        print('sendDC')
        self.dc1 = dc1
        self.dc2 = dc2

        self.sendDeviceData()

    def sendServo(self, sv1, sv2):
        print('sendServo')
        self.sv1 = sv1
        self.sv2 = sv2
        self.sendDeviceData()

    def sendDeviceData(self):
        # buffer = bytearray(15)
        buffer = struct.pack('bbbbbbbbbbbbbbb', UDP, CMD_EXECUTE,
                             self.dc1,
                             self.dc2,
                             self.sv1,
                             self.sv2,
                             self.v3,
                             self.v5,
                             self.io1,
                             self.io2,
                             self.io3,
                             self.io4, self.ultra, self.line1, self.line2)

        self.write(buffer, 15)

        # buffer[0] = UDP
        # buffer[1] = CMD_EXECUTE
        #
        # buffer[2] = self.dc1
        # buffer[3] = self.dc2
        # buffer[4] = self.sv2
        # buffer[6] = self.sv1
        #         # buffer[5] = self.v3
        # buffer[7] = self.v5
        # buffer[8] = self.io1
        # buffer[9] = self.io2
        # buffer[10] = self.io3
        # buffer[11] = self.io4
        #
        # buffer[12] = self.ultra
        # buffer[13] = self.line1
        # buffer[14] = self.line2
        #
        # self.write(bytearray(buffer), 15)

    def sendTXRX(self):
        buffer = bytearray(4)
        buffer[0] = UDP
        buffer[1] = 0xf5
        buffer[2] = 1
        buffer[3] = 'a'
        self.write(bytearray(buffer), 4)



# -*- coding:utf-8 -*-
class Protocol(object):
    """\
    Protocol as used by the ReaderThread. This base class provides empty
    implementations of all methods.
    """

    def connection_made(self, transport):
        """Called when reader thread is started"""

    def data_received(self, data):
        """Called with snippets received from the serial port"""

    def connection_lost(self, exc):
        """\
        Called when the serial port is closed or the reader loop terminated
        otherwise.
        """
        if isinstance(exc, Exception):
            raise exc



# -*- coding:utf-8 -*-
import threading




class eliochannel(threading.Thread):
    """\
    Implement a serial port read loop and dispatch to a Protocol instance (like
    the asyncio.Protocol) but do it with threads.
    Calls to close() will close the serial port but it is also possible to just
    stop() this thread and continue the serial port instance otherwise.
    """

    def __init__(self, serial_instance, protocol_factory, packet_factory):
        """\
        Initialize thread.
        Note that the serial_instance' timeout is set to one second!
        Other settings are not changed.
        """
        super(eliochannel, self).__init__()
        self.daemon = True
        self.serial = serial_instance
        self.protocol_factory = protocol_factory
        self.packet_factory = packet_factory

        self.alive = True
        self._lock = threading.Lock()
        self._connection_made = threading.Event()
        self.protocol = None
        self.packet = None

    def stop(self):
        """Stop the reader thread"""
        self.alive = False
        if hasattr(self.serial, 'cancel_read'):
            self.serial.cancel_read()
        self.join(2)

    def run(self):
        """Reader loop"""
        if not hasattr(self.serial, 'cancel_read'):
            self.serial.timeout = 1
        self.protocol = self.protocol_factory()

        self.packet = self.packet_factory(
            self.protocol.data_received, self.protocol.write_packet)
        try:
            self.protocol.connection_made(self)
        except Exception as e:
            self.alive = False
            self.protocol.connection_lost(e)
            self._connection_made.set()
            return
        error = None
        self._connection_made.set()
        first = True
        while self.alive and self.serial.is_open:
            try:
              # read all that is there or wait for one byte (blocking)
                data = self.serial.read(self.serial.in_waiting or 1)
            except serial.SerialException as e:
                # probably some I/O problem such as disconnected USB serial
                # adapters -> exit
                error = e
                break
            else:
                if data:
                    # make a separated try-except for called used code
                    try:

                        convert_buffer = bytearray(data)

                        # print(convert_buffer)
                        for ch in convert_buffer:
                            self.packet.add(ch)
                        # self.protocol.data_received(data)
                    except Exception as e:
                        error = e
                        break
        self.alive = False
        self.protocol.connection_lost(error)
        self.protocol = None

    def write(self, data):
        """Thread safe writing (uses lock)"""
        with self._lock:
            self.serial.write(data)

    def close(self):
        """Close the serial port and exit reader thread (uses lock)"""
        # use the lock to let other threads finish writing
        with self._lock:
            # first stop reading, so that closing can be done on idle port
            self.stop()
            self.serial.close()

    def connect(self):
        """
        Wait until connection is set up and return the transport and protocol
        instances.
        """
        if self.alive:
            self._connection_made.wait()
            if not self.alive:
                raise RuntimeError('connection_lost already called')
            return (self, self.protocol)
        else:
            raise RuntimeError('already stopped')

    # - -  context manager, returns protocol

    def __enter__(self):
        """\
        Enter context handler. May raise RuntimeError in case the connection
        could not be created.
        """
        self.start()
        self._connection_made.wait()
        if not self.alive:
            raise RuntimeError('connection_lost already called')
        return self.protocol

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Leave context: close port"""
        self.close()



#-*- coding:utf-8 -*-

import time
import sys

class comtimer_t:
    __m_nRetry = 0;
    __m_nMaxRetry = 0;
    __m_start=0;
    __m_timeout = 0;

    def	__init__(self):
        print('__init__')

        self.ClearRetry()
        self.SetMaxRetry(1)
        self.SetTimeout(1000)
        self.StartTimer()

    def __del__(self):
        print('__del__')

    def __millis(self):
        print('millis')
        return int(round(time.time() * 1000))

    def __StartTimer(self):
        self.m_start = self.millis();

    def DoTimerEvent(self):
        cur = self.millis();
        span = cur - self.m_start;

        if (span < 0):
            span += sys.maxint;

        if (span >= self.m_timeout):
            return False;
        return True;

    def SetTimeout(self,  ms):
        self.m_timeout = ms;

    def __SetMaxRetry(self, nMax):
        self.m_nMaxRetry = nMax;

    def __AddRetry(self):
        if (++self.m_nRetry > self.m_nMaxRetry):
            return False;
        return True;

    def __ClearRetry(self):
        self.m_nRetry = 0


# -*- coding:utf-8 -*-

import setuptools

setuptools.setup(
    name="elio",
    version="3.0.3",
    license='MIT',
    author="elio robotics",
    author_email="caram88@mobilian.biz",
    description="elio board python interface",
    long_description=open('README.md').read(),
    url="https://github.com/johnsnow-nam/elio",
    packages=setuptools.find_packages(),
    install_requires=['pyserial'],
    classifiers=[
        # 패키지에 대한 태그
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.6',
)



#-*- coding:utf-8 -*-
PS_STX = 0
PS_DATA = 1
PS_ESC = 2
MAX_PACKET_LEN = 1024
STX= 0x02
ETX = 0x03
ESC= 0x23
DLE=0x40
DEFAULT_CRC= 0xff

class packet_t(object):
    m_state = None
    m_pos = None
    m_crc = None
    m_completion_handler = None
    m_write_handler = None
    m_buffer = None

    def __init__(self, rx=None, tx=None):
        print('packet_t::__init__\n')
        self.m_buffer =bytearray(MAX_PACKET_LEN)
        self.m_completion_handler = rx
        self.m_write_handler = tx
        self.reset();

    def reset(self):
        self.m_pos   = 0
        self.m_state = PS_STX
        self.m_crc   = DEFAULT_CRC

    def write_bytes(self, ch):
        if ch <= ETX:
            self.m_write_handler(bytearray([ESC]))
            ch = ch^ DLE
            self.m_write_handler(bytearray([ch]))
        elif ch == ESC :
            self.m_write_handler(bytearray([ch]))
            self.m_write_handler(bytearray([ch]))
        else :
            self.m_write_handler(bytearray([ch]));

    def send_packet(self, buf,  len):
        if (self.m_write_handler == None):
            return;
        self.m_write_handler(bytearray([STX]));
        crc = DEFAULT_CRC
        i = 0;
        while len !=  0 :
            len -= 1
            # ch = struct.pack("B",buf[i])
            ch = buf[i]
            i += 1
            self.write_bytes(ch)
            crc ^= ch

        self.write_bytes(crc);
        self.m_write_handler(bytearray([ETX]))

    def add_data(self,  ch):
        if (self.m_pos >= MAX_PACKET_LEN):
            self.reset();
            return;

        self.m_crc ^= ch					# update crc
        self.m_buffer[self.m_pos] = ch;		#add ch to buffer
        self.m_pos+=1

    def add(self,  ch):
        if (ch == STX):
            self.reset();

        if self.m_state ==  PS_STX:
              self.m_state = PS_DATA
        elif self.m_state ==  PS_DATA:
                if (ch == ETX) :
                    self.m_state = PS_STX

                    if (self.m_pos >= 2 and self.m_crc == 0) :
                        self.m_pos-=1;
                        if (self.m_completion_handler) :
                            self.m_completion_handler(self.m_buffer, self.m_pos);
                        return;
                elif (ch == ESC):
                    self.m_state = PS_ESC;
                else:
                    if (ch < ETX):
                        self.reset();
                    else:
                        self.add_data(ch);
        elif self.m_state ==  PS_ESC:
                if (ch == ESC) :
                    self.m_state = PS_DATA;
                    self.add_data(ch);
                else:
                    self.m_state = PS_DATA;
                    self.add_data(ch ^ DLE);
        else :
            print ('')
